/* Copyright (C) 2007 Jean-Michel Richer */
#include "test_byte_func.h"
#include <stdio.h>
#include <xmmintrin.h>

/**
 * basic function : compute fitch score
 */
int func_byte_basic(char *vA, char *vB, char *vC, int len) {
  int i, f=0;

  for (i = 0; i < len; i++) {
    vC[i] = vA[i] & vB[i];
    if (vC[i] == 0) {
      vC[i] = vA[i] | vB[i];
      ++f;
    }
  }
  return f;
}

/**
 * unroll4
 *
 */
int func_byte_unroll4(char *vA, char *vB, char *vC, int len) {
  int i, f=0;
  int len4 = (len & ~3);

  for (i = 0; i < len4; i+=4) {
    vC[i]   = vA[i] & vB[i];
    if (vC[i] == 0) {
      vC[i] = vA[i] | vB[i];
      ++f;
    }
    vC[i+1] = vA[i+1] & vB[i+1];
    if (vC[i+1] == 0) {
      vC[i+1] = vA[i+1] | vB[i+1];
      ++f;
    }
    vC[i+2] = vA[i+2] & vB[i+2];
    if (vC[i+2] == 0) {
      vC[i+2] = vA[i+2] | vB[i+2];
      ++f;
    }
    vC[i+3] = vA[i+3] & vB[i+3];
    if (vC[i+3] == 0) {
      vC[i+3] = vA[i+3] | vB[i+3];
      ++f;
    }
  }
  while (i < len) {
    vC[i] = vA[i] & vB[i];
    if (vC[i] == 0) {
      vC[i] = vA[i] | vB[i];
      ++f;
    }
    ++i;
  }
  return f;
}

#define MASK1 0x000000FF
#define MASK2 0x0000FF00
#define MASK3 0x00FF0000
#define MASK4 0xFF000000

/**
 * treat by 4
 *
 */
int func_byte_by4(char *vA, char *vB, char *vC, int len) {
  int i, f=0;
  int len4 = len >> 2;
  unsigned int *uA = (unsigned int *) vA;
  unsigned int *uB = (unsigned int *) vB;
  unsigned int *uC = (unsigned int *) vC;
  unsigned int  u, v;
  
  for (i = 0; i < len4; ++i) {
    u = uA[i] & uB[i];
    v = uA[i] | uB[i];
    if ((u & MASK1)==0) {
      ++f;
      u |= (v & MASK1);
    }
    if ((u & MASK2)==0) {
      ++f;
      u |= (v & MASK2);
    }
    if ((u & MASK3)==0) {
      ++f;
      u |= (v & MASK3);
    }
    if ((u & MASK4)==0) {
      ++f;
      u |= (v & MASK4);
    }
    uC[i] = u;

    //   printf("%d %d -> %d\n", vA[i], vB[i], vC[i]);
    //printf("%d %d -> %d\n", vA[i+1], vB[i+1], vC[i+1]);
    //printf("%d %d -> %d\n", vA[i+2], vB[i+2], vC[i+2]);
    //printf("%d %d -> %d\n", vA[i+3], vB[i+3], vC[i+3]);
    
  }
  i = i << 2;
  while (i < len) {
    vC[i] = vA[i] & vB[i];
    if (vC[i] == 0) {
      vC[i] = vA[i] | vB[i];
      ++f;
    }
    ++i;
  }
  return f;
}

/**
 * xmm intrinsics : compute fitch score
 */
int func_byte_xmm_c(char *vA, char *vB, char *vC, int len) {
  int i, f=0;

  int mask;

  __m128 *xmm0 = (__m128 *) vA;
  __m128 *xmm2 = (__m128 *) vB;
  __m128 *xmm7 = (__m128 *) vC;

  __m128 xmm1, xmm3, xmm5, x_and, x_or;
  __m128i xmm4;

  len = (len >> 4);
  for (i = 0; i < len; i++) {
    xmm3 = _mm_xor_ps(xmm3, xmm3);
    xmm5 = _mm_xor_ps(xmm5, xmm5);

    x_and = _mm_and_ps(*xmm0, *xmm2);
    x_or  = _mm_or_ps(*xmm0, *xmm2);
    //    xmm4  = _mm_cmpeq_epi8(xmm4,xmm3);

    xmm4 = _mm_load_si128((const __m128i *)&x_and);

    //mask = _mm_movemask_epi8();
  }
  return f;
}
